/* Here are the AppleEvent handlers for this background-only task. */
/* Since this is a real live application, it can accept and send any */
/* Apple Events you want to send or recieve.  */

#define __AEDAES__

#include "MungeDeamon.h"

/* macro to access the low-memory global ResLoad (only for reading, there is already  */
/* a function (SetResLoad) for writing to it) */

#define mResLoad ((Boolean *)0xA5E)
extern Boolean gQuit;
extern EventRecord ERecord;
extern Boolean gHasAppleEvents;
extern short gOurResRef;


/* InitAEStuff checks for the availability of the AppleEvent Manager and */
/* installs our event handlers. */
/* if the AEM isn't around, we bail. */
void InitAEStuff(void)
{
    AEinstalls HandlersToInstall[] =  {
        {
            kCoreEventClass, kAEOpenApplication, AEOpenHandler
        },  {
            kCoreEventClass, kAEOpenDocuments, AEOpenDocHandler
        },  {
            kCoreEventClass, kAEQuitApplication, AEQuitHandler
        },  {
            kCoreEventClass, kAEPrintDocuments, AEPrintHandler
        }, 
        /* The above are the four required AppleEvents. */
        /* here are some other events in this sample */ {
            kMyEventClass, kGetResEvent, AEGetResHandler
        }
    };
    
    OSErr aevtErr = noErr;
    long aLong = 0;
    Boolean gHasAppleEvents = false;
    /* Check this machine for AppleEvents.  If they are not here (ie not 7.0)
    * then we exit */
    gHasAppleEvents = (Gestalt(gestaltAppleEventsAttr, &aLong) == noErr);
    /* The following series of calls installs all our AppleEvent Handlers.
    * These handlers are added to the application event handler list that 
    * the AppleEvent manager maintains.  So, whenever an AppleEvent happens
    * and we call AEProcessEvent, the AppleEvent manager will check our
    * list of handlers and dispatch to it if there is one.
    */
    if (gHasAppleEvents) {
        register qq;
        for (qq = 0; qq < ((sizeof(HandlersToInstall) / sizeof(AEinstalls))); qq++) {
            aevtErr = AEInstallEventHandler(HandlersToInstall[qq].theClass, HandlersToInstall[qq].theEvent,
                                            HandlersToInstall[qq].theProc, 0, false);
            if (aevtErr) {
                /* Of course, you can't tell the user why you died directly, since you have no face.  */
                /* But, you can post a notification from here to let them know. */
                ExitToShell();                              /* just fail, baby */
            }
        }
    } else {
        ExitToShell();
    }
}

/* end InitAEStuff */

void DoHighLevel(EventRecord *AERecord)
{
    /* I'm not doing any error handling here because there's not a lot */
    /* I can do, just pass the errors back. */
    AEProcessAppleEvent(AERecord);
    
}

/* end DoHighLevel */

/* This is the standard Open Application event. */
pascal OSErr AEOpenHandler(AppleEvent *messagein, AppleEvent *reply, long refIn)
{
#pragma unused (messagein,reply,refIn)
    /* we of course don't do anything here, since we're background only */
    return(noErr);
}

/* end AEOpenHandler */

/* Open Doc, opens our documents. */
/* In this case, of course, you are in the background so you should return an error */
/* here since you're not opening a document. */
/* Of course, you _might_ want to open a doc, but you will probably */
/* confuse the user if you do, since they will see no action as the */
/*  result of their clicking on a document icon. */
pascal OSErr AEOpenDocHandler(AppleEvent *messagein, AppleEvent *reply, long refIn)
{
#pragma unused (reply, refIn,messagein)
    /* we of course don't do anything here, so tell the sender that we  */
    /* didn't handle the event */
    return(errAEEventNotHandled);
}

pascal OSErr AEPrintHandler(AppleEvent *messagein, AppleEvent *reply, long refIn)
{
#pragma unused (reply,refIn,messagein)
    /* we of course don't do anything here */
    return(errAEEventNotHandled);
}

/* Standard Quit event handler, to handle a Quit event from the Finder, for example.  */
/*  DO NOT CALL EXITTOSHELL HERE  or you will never have a happy life.  */
pascal OSErr AEQuitHandler(AppleEvent *messagein, AppleEvent *reply, long refIn)
{
#pragma unused (messagein,refIn,reply)
    /*  This does _NOT_ quit, you */
    /* should NEVER quit from an AppleEvent handler.  Calling */
    /* ExitToShell here would blow things up */
    gQuit = true;
    return(noErr);
}

/* AEGetResHandler handles the event from OtherResInfo */
pascal OSErr AEGetResHandler(AppleEvent *messagein, AppleEvent *reply, long refIn)
{
#pragma unused (refIn)
    FSSpec theFSSpec;
    OSErr myErr = noErr;
    OSErr quitGetErr = noErr;
    DescType returnedType;
    Size returnedSize;
    AEDesc result;
    OSType theType;
    AEDescList theResources;
    
    /* the event carries one Alias with it as the direct parameter.  */
    /* This could (and probably should be) a list, but I wrote this kinda fast */
    /* so I didn't want to add that */
    /* the AppleEvent manager automatically coerces from Aliases to FSSpecs, which */
    /* is why I'm getting this as an FSSpec */
    
    myErr = AEGetKeyPtr(messagein, keyDirectObject, typeFSS, &returnedType, (Ptr)&theFSSpec, sizeof(FSSpec), &returnedSize);
    
    if (myErr == noErr) {
        /* get the type of resource they want to extract. */
        
        myErr = AEGetParamPtr(messagein, kMyResTypeParam, typeMyResType, &returnedType, (Ptr)&theType, sizeof(OSType),
                              &returnedSize);
        if (myErr == noErr) {

            /* create a list to pass to the extractor routine */
            myErr = AECreateList(nil, nil, false, &theResources);
            if (myErr == noErr) {

                myErr = OpenAndExtractResource(&theFSSpec, &theResources, theType);
                if (myErr == noErr || myErr == kAppleEventTooBig ) {
                    /* put the returned data in the reply */
                    AEPutKeyDesc(reply, keyDirectObject, &theResources);
                    AEDisposeDesc(&theResources);
                }
				if(myErr){
				/* add error information for this failure */
				long longErr = myErr;
				myErr = AEPutParamPtr(reply, keyErrorNumber, typeLongInteger, (Ptr)&longErr, sizeof(long));            
				/* In this case, I'm quitting if I get an error */
				/* again, I'm setting the quit flag, not Xiting to shell, since I am  */
				/* still working inside an AppleEvent handler */
				gQuit = true;
				}
            }                                               /* list create error */
        }                                                   /* get param ptr error */
    }                                                       /* key ptr error */
    /* one more thing to do here.  I'm also adding a 'quit' attribute to the event, if */
    /* this attribute is set then I will set my Quit flag as I exit the handler.  */
    /* This allows me to not worry about sending a Quit event, a savings of a little bit of code. */
    quitGetErr = AEGetKeyDesc(messagein, keyMyQuitFlag, typeWildCard, &result);
    /* now I don't really care what the data is, all I care bout is if it exists.  So... */
    if (quitGetErr == noErr) {
        AEDisposeDesc(&result);
        gQuit = true;
    }
    return(myErr);
    
}

/* OpenAndExtractResource is where I do the actual opening of the file and the extraction of the  */
/* resource information I'll pass back to the other application */

OSErr OpenAndExtractResource(FSSpecPtr theFSSpec, AEDescList *passedList, ResType theType)
{
    Boolean weOpened = false;                               /* so we know if the file was already opened when we started */
    short resRef = 0;
    short index;
	short typeIndex=1;
    short count = 0;
    register qq;
    OSErr myErr = noErr;
    Handle theResource;
    Handle returnedData = NewHandle(0);
    Boolean oldResLoad;
    CInfoPBPtr ourCatInfo = nil;
	Boolean gettingAll = false;
    myResInfoPtr currentResInfo = NewPtrClear(sizeof(myResInfoStruct));
	
	/* see if the user wants information on all resources in the file */
	if(theType == typeWildCard)gettingAll = true;
	
    ourCatInfo = (CInfoPBPtr)NewPtrClear(sizeof(CInfoPBRec));
    /* first try and open it for Writing.  If we can do this, that means we can  */
    /* close it ourselves also */
    /* we'll look at the finder bit to see if it's already open by somebody  */
    /* else.  I'm looking specifically at the resource fork. */
    /* did we get memory? */
    if (ourCatInfo != nil) {
        ourCatInfo->hFileInfo.ioVRefNum = theFSSpec->vRefNum;
        ourCatInfo->hFileInfo.ioDirID = theFSSpec->parID;
        ourCatInfo->hFileInfo.ioNamePtr = &(theFSSpec->name);
        PBGetCatInfo(ourCatInfo, false);
        /* see if the file's resource fork is already open or nut */
        if (ourCatInfo->hFileInfo.ioFlAttrib & 0x4)
            weOpened = false;
        else
            weOpened = true;
        DisposePtr((Ptr)ourCatInfo);
        ourCatInfo = nil;
    } else {
        /* if we didn't have the memory to do the PBGetCat then we have more troubles than I'd */
        /* like to think about, but I'm gonna press on anyway, just set my opened flag to false */
        weOpened = false;
        
    }
    if (currentResInfo == nil) {
        /* hmmmm, we didn't get the pointer memory.  Bail out */
        myErr = dsMemFullErr;
    }
    if (myErr == noErr) {
		/* We do NOT want to actually load any of the resources in */
		/* the file we're investigating! */
		/* So, we set ResLoad to be False. */
		/* Note that I am setting ti to false BEFORE I even open the file */
		/* If you don't do that then all the resources marked 'preload' */
		/* will be loaded into your heap as soon as you open this fork */
		oldResLoad = *mResLoad;
        SetResLoad(false);
		resRef = FSpOpenResFile(theFSSpec, fsRdPerm);
        
        if (resRef != -1) {
            /* it's opened, grab all the resources of the requested type */
			
			/* make sure we're getting at the resource file we want */            
            UseResFile(resRef);                             
            

			/* see if they want everything  */            
			if(gettingAll)
				typeIndex = Count1Types();
			
				/* if they didn't want all the types, typeIndex was set to 1 */
				/* as this routine was initialized, so we'll use it as an   */
				/* index anyway */
			do{				
            
			/* count the types that the folks are looking for. */
			/* ONLY if they want all.  Else, just the type they requested */
			
			if(gettingAll)Get1IndType(&theType,typeIndex);
            
			index = Count1Resources(theType);
            
            /* loop through them all  */
            for (qq = 1; qq < index + 1; qq++) {
                
                /* get the nth one */
                theResource = Get1IndResource(theType, qq);
                
                if (theResource) {
                    
                    /* Add this resource's info  to our list*/
                    
                    /* our block is already set up, of course */
                    GetResInfo(theResource, &currentResInfo->id, &currentResInfo->theType, &currentResInfo->theName);
                    currentResInfo->length = SizeResource(theResource);
                    
                    
                    /* and put it in our list  */
                    
					/* NOTE: I am Pre-Flighing my addition to the list here!!!!! */
					/* AppleEvents can only be 64k currently, so if my list is over */
					/* 60k, I bail out of this mess and pass back what I have already collected */
					/* instead of blowing up because of limited memory */
					/* I'm leaving 4k of scratch space for the rest of the event (like address, */
					/* transaction ID, and ullage (as we used to say in the Hydraulic world)) */
					if(GetHandleSize(passedList->dataHandle) > 60000){
					myErr = kAppleEventTooBig;
					} else {
					
					myErr = AEPutPtr(passedList, 0, typeMyResInfoType, (Ptr)currentResInfo, sizeof(myResInfoStruct));
					        }            
                    /* if we get any errors, bail out of here.  In all likelyhood it'll be  */
                    /* a memory error */
                    if (myErr)
                        break;
                }
            }
			typeIndex --;
        }while(typeIndex && myErr == noErr);
		}
        
        /* reset resLoad to what it should be  */
        SetResLoad(oldResLoad);
        
        /* Reset the current resource file to Us, in case we can't close the file, we don't */
        /* want other resources loading in place of ours */
        /* This is MANDITORY if you are not quitting at once. */
        /* You may want to just default to 'Quit after action', that is the  */
        /* safest course to prevent resource map confusion */
        UseResFile(gOurResRef);
        /* kill our memory  */
        DisposePtr((Ptr)currentResInfo);
        
        /* if we were the only ones who had this file open, then we can close it */
        if (weOpened)
            CloseResFile(resRef);
        if (resRef == -1)
            myErr = openErr;
    }                                                       /* mem error if  */
    return(myErr);
}


#undef __AEDAES__
